/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup debug
 * @{
 */
/** @file
 */

#include <stdio.h>
#include <stdlib.h>
#include <fibril.h>
#include <async.h>
#include <futex.h>

#include "fib_synch.h"

void fcv_init(fcv_t *fcv)
{
	int i;

	for (i = 0; i < MAX_FIB; i++) {
		fcv->waiting[i] = 0;
	}
}

void fcv_wait(fcv_t *fcv)
{
	fid_t fid;
	int i;

	futex_down(&async_futex);

	fid = fibril_get_id();

	for (i = 0; i < MAX_FIB; i++) {
		if (fcv->waiting[i] == 0) {		
			fcv->waiting[i] = fid;

			fibril_switch(FIBRIL_TO_MANAGER);
			futex_up(&async_futex);
			
			return;
		}
	}

	futex_up(&async_futex);
	printf("fcv_failed: too many waiting fibrils\n");
	exit(1);
}

void fcv_broadcast(fcv_t *fcv)
{
	int i;

	futex_down(&async_futex);

	for (i = 0; i < MAX_FIB; i++) {
		if (fcv->waiting[i] != 0) {
			fibril_add_ready(fcv->waiting[i]);
			fcv->waiting[i] = 0;
		}
	}

	futex_up(&async_futex);
}

/** @}
 */
